------------------------------------------------------------------------------
--               PostgreSQL server extension modules binding                --
--                                                                          --
--                       Copyright (C) 2020, AdaCore                        --
--                                                                          --
-- This library is free software;  you can redistribute it and/or modify it --
-- under terms of the  GNU General Public License  as published by the Free --
-- Software  Foundation;  either version 3,  or (at your  option) any later --
-- version. This library is distributed in the hope that it will be useful, --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE.                            --
--                                                                          --
-- As a special exception under Section 7 of GPL version 3, you are granted --
-- additional permissions described in the GCC Runtime Library Exception,   --
-- version 3.1, as published by the Free Software Foundation.               --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
------------------------------------------------------------------------------
--  Calling conventions version declarations subprogram

package PGXS.ABI is

   pragma Preelaborate;

   type Function_Info_Record is limited private;

   function Ada_Function_Info_1
     return not null access PGXS.ABI.Function_Info_Record
     with Import, Convention => C, Link_Name  => "pg_finfo__ada_function";
   --  Returns version infomation for the calling conventions version 1.
   --  For calling conventions version 1 each user defined function should
   --  have additional exported function which should call this function.
   --  See documentation for more information.

private

   type Function_Info_Record is limited null record
     with Convention => C;

end PGXS.ABI;
