------------------------------------------------------------------------------
--               PostgreSQL server extension modules binding                --
--                                                                          --
--                       Copyright (C) 2020, AdaCore                        --
--                                                                          --
-- This library is free software;  you can redistribute it and/or modify it --
-- under terms of the  GNU General Public License  as published by the Free --
-- Software  Foundation;  either version 3,  or (at your  option) any later --
-- version. This library is distributed in the hope that it will be useful, --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE.                            --
--                                                                          --
-- As a special exception under Section 7 of GPL version 3, you are granted --
-- additional permissions described in the GCC Runtime Library Exception,   --
-- version 3.1, as published by the Free Software Foundation.               --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
------------------------------------------------------------------------------
--  Wrapper around BYTEA to represent its value as Ada record

with PGXS.Types;

generic
   type Data is private;

package PGXS.Generic_Bytea is

   function Get_Arg
     (Args : Function_Call_Info; Index : PGXS.Types.Int_32) return Data;
   --  Returns value of the function's argument as object of user defined type.
   --  Caller is responsible to check 'null' value of the parameter first.

   function Return_Value
     (Args : Function_Call_Info; Item : Data) return PGXS.Datum;
   --  Construct bytea value from the given object of the user defined type
   --  and return it as Datum.

end PGXS.Generic_Bytea;
